import { Provide, Inject } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { BaseService, ServiceError } from 'free-midway-component';
import { Course } from '../../entity/core/course.entity';
import { get } from 'lodash';
import { Teacher } from '../../entity/core/teacher.entity';
import { I18nService } from '../i18n.service';
import { FrameErrorEnum } from '../../error/frame.error';
type modelType = ReturnModelType<typeof Course>;
@Provide()
export class CourseService extends BaseService<modelType> {
  @InjectEntityModel(Course)
  model: modelType;

  @Inject()
  i18n: I18nService;

  @InjectEntityModel(Teacher)
  tModel: ReturnModelType<typeof Teacher>;

  // 创建前判断改教师是否审核成功
  async createBefore(data) {
    const { teacher } = data;
    const teacherInfo = await this.tModel.findById(teacher).lean();
    if (teacherInfo && teacherInfo._id) {
      if (teacherInfo.status === '0') throw new ServiceError(this.i18n.translateError(FrameErrorEnum.TEACHER_STATUS), FrameErrorEnum.TEACHER_STATUS);
      if (teacherInfo.is_show === '1') throw new ServiceError(this.i18n.translateError(FrameErrorEnum.TEACHER_SHOW), FrameErrorEnum.TEACHER_SHOW);
    } else throw new ServiceError(this.i18n.translateError(FrameErrorEnum.USER_NOT_FOUND), FrameErrorEnum.USER_NOT_FOUND);
  }

  // 列表
  async list(query) {
    const { skip = 0, limit = 0, ...info } = query;
    const data = await this.model.find(info, { meta: 0, __v: 0 }).skip(skip).limit(limit).lean();
    for (const val of data) {
      if (get(val, 'teacher')) {
        const userData = await this.tModel.findById(val.teacher).lean();
        if (userData) Object.assign(val, { teacher_name: userData.nick_name, teacher_phone: userData.phone });
      }
    }
    const total = await this.model.count(info);
    return { data, total };
  }
  // 小程序首页显示课程
  async appHome(query) {
    const { skip = 0, limit = 0, ...info } = query;
    const data = [];
    const list = await this.model.find(info, { meta: 0, __v: 0 }).skip(skip).limit(limit).sort({ start_time: -1 }).lean();
    for (const val of list) {
      const info = { _id: val._id, name: val.name, status: val.status, money: val.money, start_time: val.start_time, teacher: val.teacher };
      if (get(val, 'teacher')) {
        const userData = await this.tModel.findById(val.teacher).lean();
        if (userData) Object.assign(info, { teacher_name: userData.nick_name, icon: userData.icon });
      }
      data.push(info);
    }
    const total = await this.model.count(info);
    return { data, total };
  }
  // 小程序首页显示详情
  async appHomeDetail(id) {
    const result = await this.model.findById(id).lean();
    const info = { courseInfo: result };
    if (get(result, 'teacher')) {
      const userData = await this.tModel.findById(result.teacher).lean();
      if (userData) Object.assign(info, { teacherInfo: userData });
    }
    return info;
  }
}
